@@ -21,6 +21,7 @@
 
 #include <assert.h>
 #include <math.h>
+#include "pow-24.h"
 
 /* Alpha threshold used in the reference implementation for
  * un-pre-multiplication of color data:
@@ -55,38 +56,50 @@
 #define BABL_USE_SRGB_GAMMA
 
 #ifdef BABL_USE_SRGB_GAMMA
-
-/*  fast approximation of x ^ (5/12) from a response by David Hammen at
- *  http://stackoverflow.com/questions/6475373/optimizations-for-pow-with-const-non-integer-exponent/6475516#6475516
- *
- */
-static inline double xpow512 (double x)
-{
-  double cbrtx = cbrt(x);
-  return cbrtx*sqrt(sqrt(cbrtx));
-}
-
-
 static inline double
 linear_to_gamma_2_2 (double value)
 {
-#if 0
-  if (value > 0.0030402477F)
-    return 1.055F * pow (value, (1.0F/2.4F)) - 0.055F;
-  return 12.92F * value;
-#else
-  if (value > 0.0030402477F)
-    return 1.055F * xpow512 (value) - 0.055F;
-  return 12.92F * value;
-#endif
+  if (value > 0.003130804954)
+    return 1.055 * pow (value, (1.0/2.4)) - 0.055;
+  return 12.92 * value;
 }
 
 static inline double
 gamma_2_2_to_linear (double value)
 {
-  if (value > 0.03928F)
-    return pow ((value + 0.055F) / 1.055F, 2.4F);
-  return value / 12.92F;
+  if (value > 0.04045)
+    return pow ((value + 0.055) / 1.055, 2.4);
+  return value / 12.92;
+}
+static inline double
+babl_linear_to_gamma_2_2 (double value)
+{
+  if (value > 0.003130804954)
+    return 1.055 * babl_pow_1_24 (value) - 0.055;
+  return 12.92 * value;
+}
+static inline float
+babl_linear_to_gamma_2_2f (float value)
+{
+  if (value > 0.003130804954f)
+    return 1.055f * babl_pow_1_24f (value) - 0.055f;
+  return 12.92f * value;
+}
+
+
+static inline double
+babl_gamma_2_2_to_linear (double value)
+{
+  if (value > 0.04045)
+    return babl_pow_24 ((value + 0.055) / 1.055);
+  return value / 12.92;
+}
+static inline float
+babl_gamma_2_2_to_linearf (float value)
+{
+  if (value > 0.04045f)
+    return babl_pow_24f ((value + 0.055f) / 1.055f);
+  return value / 12.92f;
 }
 
 #else
