@@ -28,7 +28,7 @@
 
 #define INLINE inline
 
-#define  LSHIFT 2
+#define  LSHIFT 4
 
 typedef  float (* BablLookupFunction) (float  value,
                                        void  *data);
@@ -39,8 +39,9 @@ typedef struct BablLookup
   BablLookupFunction function;
   void              *data;
   int               shift;
-  uint32_t            positive_min, positive_max, negative_min, negative_max;
-  uint32_t            bitmask[babl_LOOKUP_MAX_ENTRIES/32];
+  uint32_t          positive_min, positive_max, negative_min, negative_max;
+  uint32_t          bitmask[babl_LOOKUP_MAX_ENTRIES/32];
+  int               entries;
   float             table[];
 } BablLookup;
 
@@ -54,44 +55,73 @@ static BablLookup *babl_lookup_new (BablLookupFunction  function,
 static void        babl_lookup_free      (BablLookup         *lookup);
 #endif
 
+#include <string.h>
 
 static inline float
 babl_lookup (BablLookup *lookup,
-             float      number)
+             float       number)
 {
-  union
-  {
-    float   f;
-    uint32_t i;
-  } u;
+  union { float   f; uint32_t i; } u;
+  union { float   f; uint32_t i; } ub;
+  union { float   f; uint32_t i; } ua;
+
   uint32_t i;
+  float dx = 0.0;
 
   u.f = number;
-  i = (u.i << LSHIFT )>> lookup->shift;
+  i = (u.i << LSHIFT ) >> lookup->shift;
+
+  if (i > lookup->positive_min && i < lookup->positive_max)
+  {
+    ua.i = ((i) << lookup->shift)    >> LSHIFT;
+    ub.i = ((i+ 1) << lookup->shift) >> LSHIFT;
 
-  if (i > lookup->positive_min &&
-      i < lookup->positive_max)
     i = i - lookup->positive_min;
-  else if (i > lookup->negative_min &&
-           i < lookup->negative_max)
+  }
+  else if (i > lookup->negative_min && i < lookup->negative_max)
+  {
+
+    ua.i = ((i) << lookup->shift)    >> LSHIFT;
+    ub.i = ((i+ 1) << lookup->shift) >> LSHIFT;
+
     i = i - lookup->negative_min + (lookup->positive_max - lookup->positive_min);
+  }
   else
+  {
     return lookup->function (number, lookup->data);
+  }
+
+  {
+    uint32_t bm =u.i & 0b11110000000000000000000000000000;
+    ua.i |= bm;
+    ub.i |= bm;
+  }
+  dx = (u.f-ua.f) / (ub.f - ua.f);
+
+  {
 
   if (!(lookup->bitmask[i/32] & (1UL<<(i & 31))))
     {
-      /* XXX: should look up the value in the middle of the range
-       *      that yields a given value,
-       *
-       *      potentially even do linear interpolation between
-       *      the two neighbour values to get away with a tiny
-       *      lookup table.. 
-       */
-      lookup->table[i]= lookup->function (number, lookup->data);
+      lookup->table[i]= lookup->function (ua.f, lookup->data);
+      lookup->bitmask[i/32] |= (1UL<<(i & 31));
+    }
+  i++;
+  if (i< lookup->entries-2)
+  {
+    if (!(lookup->bitmask[i/32] & (1UL<<(i & 31))))
+    {
+      lookup->table[i]= lookup->function (ub.f, lookup->data);
       lookup->bitmask[i/32] |= (1UL<<(i & 31));
     }
 
-  return lookup->table[i];
+    return lookup->table[i-1] * (1.0f-dx) +
+           lookup->table[i] * (dx);
+  }
+  else
+  {
+    return lookup->table[i-1];
+  }
+  }
 }
 
 static BablLookup *
@@ -124,10 +154,12 @@ babl_lookup_new (BablLookupFunction function,
   else if (precision <= 0.000040) shift = 10;
   else if (precision <= 0.000081) shift = 11;
   else if (precision <= 0.000161) shift = 12;
+  else if (precision <= 0.000200) shift = 13;
   else if (precision <= 0.000324) shift = 14;
   else if (precision <= 0.000649) shift = 15;
   else shift = 16; /* a bit better than 8bit sRGB quality */
 
+
   /* Adjust slightly away from 0.0, saving many entries close to 0, this
    * causes lookups very close to zero to be passed directly to the
    * function instead.
@@ -215,6 +247,9 @@ babl_lookup_new (BablLookupFunction function,
   lookup->function = function;
   lookup->data = data;
 
+  lookup->entries = (positive_max-positive_min)+
+                    (negative_max-negative_min);
+
   return lookup;
 }
 
@@ -222,7 +257,7 @@ static BablLookup *fast_pow = NULL;
 
 static inline float core_lookup (float val, void *userdata)
 {
-  return linear_to_gamma_2_2 (val);
+  return babl_linear_to_gamma_2_2f (val);
 }
 
 static float
@@ -231,12 +266,11 @@ linear_to_gamma_2_2_lut (float val)
   return babl_lookup (fast_pow, val);
 }
 
-
 static BablLookup *fast_rpow = NULL;
 
 static inline float core_rlookup (float val, void *userdata)
 {
-  return gamma_2_2_to_linear (val);
+  return babl_gamma_2_2_to_linearf (val);
 }
 
 static float
@@ -264,15 +298,164 @@ conv_rgbaF_linear_rgbAF_gamma (unsigned char *src,
 
    while (n--)
      {
-       float alpha = fsrc[3];
-       *fdst++ = linear_to_gamma_2_2_lut (*fsrc++) * alpha;
-       *fdst++ = linear_to_gamma_2_2_lut (*fsrc++) * alpha;
-       *fdst++ = linear_to_gamma_2_2_lut (*fsrc++) * alpha;
-       *fdst++ = *fsrc++;
+       float red   = *fsrc++;
+       float green = *fsrc++;
+       float blue  = *fsrc++;
+       float alpha = *fsrc++;
+       if (alpha == 1.0)
+       {
+         *fdst++ = linear_to_gamma_2_2_lut (red);
+         *fdst++ = linear_to_gamma_2_2_lut (green);
+         *fdst++ = linear_to_gamma_2_2_lut (blue);
+         *fdst++ = alpha;
+       }
+       else if (alpha == 0.0)
+       {
+         *fdst++ = 0.0;
+         *fdst++ = 0.0;
+         *fdst++ = 0.0;
+         *fdst++ = 0.0;
+       }
+       else
+       {
+         *fdst++ = linear_to_gamma_2_2_lut (red)   * alpha;
+         *fdst++ = linear_to_gamma_2_2_lut (green) * alpha;
+         *fdst++ = linear_to_gamma_2_2_lut (blue)  * alpha;
+         *fdst++ = alpha;
+       }
      }
   return samples;
 }
 
+
+
+static INLINE long
+conv_rgbaF_linear_rgba8_gamma (unsigned char *src, 
+                               unsigned char *dst, 
+                               long           samples)
+{
+   float *fsrc = (float *) src;
+   uint8_t *cdst = (uint8_t *) dst;
+   int n = samples;
+
+   while (n--)
+     {
+       float red   = *fsrc++;
+       float green = *fsrc++;
+       float blue  = *fsrc++;
+       float alpha = *fsrc++;
+       if (alpha <= 0) /* XXX: we need to drop alpha!! ? */
+       {
+       *cdst++ = 0;
+       *cdst++ = 0;
+       *cdst++ = 0;
+       *cdst++ = 0;
+       }
+       else
+       {
+       int val = linear_to_gamma_2_2_lut (red) * 0xff + 0.5f;
+       *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+       val = linear_to_gamma_2_2_lut (green) * 0xff + 0.5f;
+       *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+       val = linear_to_gamma_2_2_lut (blue) * 0xff + 0.5f;
+       *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+       val = alpha * 0xff + 0.5;
+       *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+       }
+     }
+  return samples;
+}
+
+static INLINE long
+conv_rgbaF_linear_rgbA8_gamma (unsigned char *src, 
+                               unsigned char *dst, 
+                               long           samples)
+{
+   float *fsrc = (float *) src;
+   uint8_t *cdst = (uint8_t *) dst;
+   int n = samples;
+
+   while (n--)
+     {
+       float red   = *fsrc++;
+       float green = *fsrc++;
+       float blue  = *fsrc++;
+       float alpha = *fsrc++;
+       if (alpha >= 1.0)
+       {
+         int val = linear_to_gamma_2_2_lut (red) * 0xff + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         val = linear_to_gamma_2_2_lut (green) * 0xff + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         val = linear_to_gamma_2_2_lut (blue) * 0xff + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         *cdst++ = 0xff;
+       }
+       else if (alpha <= 0.0)
+       {
+         *((uint32_t*)(cdst))=0;
+	     cdst+=4;
+       }
+       else
+       {
+         float balpha = alpha * 0xff;
+         int val = linear_to_gamma_2_2_lut (red) * balpha + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         val = linear_to_gamma_2_2_lut (green) * balpha + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         val = linear_to_gamma_2_2_lut (blue) * balpha + 0.5f;
+         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+         *cdst++ = balpha + 0.5f;
+       }
+     }
+  return samples;
+}
+
+static long
+conv_rgbaF_linear_rgbA8_gamma_cairo (unsigned char *src, 
+                                     unsigned char *dst, 
+                                     long           samples)
+{
+  float *fsrc = (float *) src;
+  unsigned char *cdst = (unsigned char *) dst;
+  int n = samples;
+
+  while (n--)
+    {
+      float red   = *fsrc++;
+      float green = *fsrc++;
+      float blue  = *fsrc++;
+      float alpha = *fsrc++;
+      if (alpha >= 1.0)
+      {
+        int val = linear_to_gamma_2_2_lut (blue) * 0xff + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        val = linear_to_gamma_2_2_lut (green) * 0xff + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        val = linear_to_gamma_2_2_lut (red) * 0xff + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        *cdst++ = 0xff;
+      }
+      else if (alpha <= 0.0)
+      {
+        *((uint32_t*)(cdst))=0;
+        cdst+=4;
+      }
+      else
+      {
+        float balpha = alpha * 0xff;
+        int val = linear_to_gamma_2_2_lut (blue) * balpha + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        val = linear_to_gamma_2_2_lut (green) * balpha + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        val = linear_to_gamma_2_2_lut (red) * balpha + 0.5f;
+        *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
+        *cdst++ = balpha + 0.5f;
+      }
+    }
+  return samples;
+}
+
 static INLINE long
 conv_rgbAF_linear_rgbAF_gamma (unsigned char *src, 
                                unsigned char *dst, 
@@ -284,29 +467,31 @@ conv_rgbAF_linear_rgbAF_gamma (unsigned char *src,
 
    while (n--)
      {
-       float alpha = fsrc[3];
+      float red   = *fsrc++;
+      float green = *fsrc++;
+      float blue  = *fsrc++;
+      float alpha = *fsrc++;
        if (alpha < BABL_ALPHA_THRESHOLD)
          {
            *fdst++ = 0.0;
            *fdst++ = 0.0;
            *fdst++ = 0.0;
            *fdst++ = 0.0;
-           fsrc+=4;
          }
        else if (alpha >= 1.0)
          {
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++);
+           *fdst++ = linear_to_gamma_2_2_lut (red);
+           *fdst++ = linear_to_gamma_2_2_lut (green);
+           *fdst++ = linear_to_gamma_2_2_lut (blue);
            *fdst++ = *fsrc++;
          }
        else
          {
            float alpha_recip = 1.0 / alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++ * alpha_recip) * alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++ * alpha_recip) * alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (*fsrc++ * alpha_recip) * alpha;
-           *fdst++ = *fsrc++;
+           *fdst++ = linear_to_gamma_2_2_lut (red   * alpha_recip) * alpha;
+           *fdst++ = linear_to_gamma_2_2_lut (green * alpha_recip) * alpha;
+           *fdst++ = linear_to_gamma_2_2_lut (blue  * alpha_recip) * alpha;
+           *fdst++ = alpha;
          }
      }
   return samples;
@@ -427,6 +612,25 @@ init (void)
     babl_component ("B'a"),
     babl_component ("A"),
     NULL);
+
+  const Babl *rgbA8_gamma = babl_format_new (
+    babl_model ("R'aG'aB'aA"),
+    babl_type ("u8"),
+    babl_component ("R'a"),
+    babl_component ("G'a"),
+    babl_component ("B'a"),
+    babl_component ("A"),
+    NULL);
+
+  const Babl *rgba8_gamma = babl_format_new (
+    babl_model ("R'G'B'A"),
+    babl_type ("u8"),
+    babl_component ("R'"),
+    babl_component ("G'"),
+    babl_component ("B'"),
+    babl_component ("A"),
+    NULL);
+
   const Babl *rgbF_linear = babl_format_new (
     babl_model ("RGB"),
     babl_type ("float"),
@@ -446,10 +650,11 @@ init (void)
     float f;
     float a;
 
-    fast_pow = babl_lookup_new (core_lookup, NULL, 0.0, 1.0,   0.0001);
-    fast_rpow = babl_lookup_new (core_rlookup, NULL, 0.0, 1.0, 0.0001);
+    /* tweaking the precision - does impact speed.. */
+    fast_pow = babl_lookup_new (core_lookup, NULL, 0.0, 1.0,   0.000199);
+    fast_rpow = babl_lookup_new (core_rlookup, NULL, 0.0, 1.0, 0.000250);
 
-    for (f = 0.0; f < 1.0; f+= 0.00001)
+    for (f = 0.0; f < 1.0; f+= 0.0000001)
       {
         a = linear_to_gamma_2_2_lut (f);
         a = gamma_2_2_to_linear_lut (f);
@@ -459,12 +664,31 @@ init (void)
 
   }
 
+
+  {
+     const Babl *f32 = babl_format_new (
+        "name", "cairo-ARGB32",
+        babl_model ("R'aG'aB'aA"),
+        babl_type ("u8"),
+        babl_component ("B'a"),
+        babl_component ("G'a"),
+        babl_component ("R'a"),
+        babl_component ("A"),
+        NULL
+      );
+
+
+    babl_conversion_new (rgbaF_linear, f32, "linear", conv_rgbaF_linear_rgbA8_gamma_cairo, NULL);
+  }
+
+  o (rgbaF_linear, rgbA8_gamma);
   o (rgbAF_linear, rgbAF_gamma);
   o (rgbaF_linear, rgbAF_gamma);
   o (rgbaF_linear, rgbaF_gamma);
+  o (rgbaF_linear, rgba8_gamma);
   o (rgbaF_gamma,  rgbaF_linear);
-  o (rgbF_linear, rgbF_gamma);
-  o (rgbF_gamma,  rgbF_linear);
+  o (rgbF_linear,  rgbF_gamma);
+  o (rgbF_gamma,   rgbF_linear);
 
   return 0;
 }
