@@ -16,6 +16,7 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
 #include <string.h>
 #include <stdio.h>
 #include "babl-internal.h"
@@ -68,6 +69,6 @@ main (int    argc,
   babl_init ();
   if (test ())
     return -1;
-  babl_destroy ();
+  babl_exit ();
   return 0;
 }
